==> fusenshi-0.8.2.tgz: fusenshi-0.8.2/src/net/fusenshisslclientsocket.cpp <==
/*
 * $Id: fusenshisslclientsocket.cpp,v 1.1 2005/12/02 07:08:00 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.1 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "fusenshisslclientsocket.h"

#include "../crypto/selfcertificate.h"

FusenshiSSLClientSocket::FusenshiSSLClientSocket()
{
    // SSL
    // ̿Ź沽ޤۥȤξϹԤޤ
    
    SSL_library_init();
    OpenSSL_add_all_algorithms();
    SSL_load_error_strings();

    m_ctx = NULL;
    m_ssl = NULL;
}

FusenshiSSLClientSocket::~FusenshiSSLClientSocket()
{
    close();
}

bool FusenshiSSLClientSocket::connect(unsigned long ip, int port, struct timeval& timeout)
{
    if (NULL != m_ssl) close();
    
    if (FusenshiClientSocket::connect(ip, port, timeout)) {
        X509* x509 = SelfCertificate::getSelfCertificate();
        if (!x509) return false;
        
        EVP_PKEY* pkey = SelfCertificate::getSelfCertificatePKey();
        if (!pkey) return false;

        m_ctx = SSL_CTX_new(SSLv23_client_method());
        if (!m_ctx) {
            ERR_print_errors_fp(stderr);
            close();
            return false;
        }

        m_ssl = SSL_new(m_ctx);
        if (!m_ssl) {
            ERR_print_errors_fp(stderr);
            close();
            return false;
        }

        if (!SSL_set_fd(m_ssl, m_socket)) {
            ERR_print_errors_fp(stderr);
            close();
            return false;
        }

        SSL_use_certificate(m_ssl, x509);
        SSL_use_PrivateKey(m_ssl, pkey);
        SSL_set_verify(m_ssl, SSL_VERIFY_NONE, NULL);

        if (1 != SSL_connect(m_ssl)) {
            ERR_print_errors_fp(stderr);
            close();
            return false;
        }
        
        return true;
    }
    
    return false;
}

ssize_t FusenshiSSLClientSocket::recv(void* buf, size_t len)
{
    return SSL_read(m_ssl, buf, len);
}

ssize_t FusenshiSSLClientSocket::send(const void* buf, size_t len)
{
    return SSL_write(m_ssl, buf, len);
}

int FusenshiSSLClientSocket::shutdown()
{
    if (!m_ssl || 1 == SSL_shutdown(m_ssl)) {
        return FusenshiSocket::shutdown();
    }
    
    return -1;
}

int FusenshiSSLClientSocket::close()
{
    shutdown();

    if (m_ssl) {
        SSL_free(m_ssl);
        m_ssl = NULL;
    }
    if (m_ctx) {
        SSL_CTX_free(m_ctx);
        m_ctx = NULL;
    }

    return FusenshiSocket::close();
}

bool FusenshiSSLClientSocket::operator()()
{
    return (NULL != m_ssl);
}

==> fusenshi-0.9.0.tgz: fusenshi-0.9.0/src/net/fusenshisslclientsocket.cpp <==
/*
 * $Id: fusenshisslclientsocket.cpp,v 1.2 2007/04/02 10:44:52 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.2 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "fusenshisslclientsocket.h"

#include "../crypto/selfcertificate.h"

FusenshiSSLClientSocket::FusenshiSSLClientSocket()
{
    m_session = NULL;
}

FusenshiSSLClientSocket::~FusenshiSSLClientSocket()
{
    close();
}

bool FusenshiSSLClientSocket::connect(unsigned long ip, int port, struct timeval& timeout)
{
    // ̿Ź沽ޤۥȤξϹԤޤ
    int ret;
	const int cert_priority[] = {
		GNUTLS_CRT_X509, 0
	};

    if (NULL != m_session) close();
    
    if (FusenshiClientSocket::connect(ip, port, timeout)) {
        gnutls_certificate_credentials_t cert = SelfCertificate::getSelfCertificate();
        if (!cert) return false;
        
		gnutls_init(&m_session, GNUTLS_CLIENT);
		gnutls_set_default_priority(m_session);
		gnutls_certificate_type_set_priority(m_session, cert_priority);

		gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, cert);
		
		gnutls_transport_set_ptr(m_session, (gnutls_transport_ptr_t)m_socket);
		ret = gnutls_handshake(m_session);
		if (ret < 0) {
			gnutls_deinit(m_session);
			m_session = NULL;
			fprintf(stderr, "%s: SSL/TLS handshake has failed. (%s)\n", APP_NAME, gnutls_strerror(ret));
            return false;
        }

        return true;
    }
    
    return false;
}

ssize_t FusenshiSSLClientSocket::recv(void* buf, size_t len)
{
    return gnutls_record_recv(m_session, buf, len);
}

ssize_t FusenshiSSLClientSocket::send(const void* buf, size_t len)
{
    return gnutls_record_send(m_session, buf, len);
}

int FusenshiSSLClientSocket::shutdown()
{
    if (m_session)
    	gnutls_bye(m_session, GNUTLS_SHUT_RDWR);
    else
        return FusenshiSocket::shutdown();
    
    return 0;
}

int FusenshiSSLClientSocket::close()
{
    shutdown();
    
    int result = FusenshiSocket::close();

    if (m_session) {
        gnutls_deinit(m_session);
        m_session = NULL;
    }

    return result;
}

bool FusenshiSSLClientSocket::operator()()
{
    return (NULL != m_session);
}
